#!/usr/bin/perl
# Id: hellosign.pl,v 1.1 2011/02/05 21:09:49 rjs Exp 
#
# PDFlib PLOP DS:
# generate a dynamic visualization document in memory with PDFlib,
# and digitally sign a second document, using the dynamic document
# for signature visualization.

from sys import *
from traceback import *
from PDFlib.PLOP import *

from PDFlib.PDFlib import *
import time

  
# Basic signature options

# use ltv=full to ensure LTV status
# for  use with PKCS11
# "engine=pkcs#11 digitalid={filename=cryptoki.dll} "
sign_opts = \
    "ltv=try " \
    "engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} password={demo} "

   
# This is where input files live. Adjust as necessary.
searchpath = "../data"

if len(argv) != 3 :
    raise Exception("usage: dynamicsign <infile> <outfile>" )

plop = None
p = None
try:
    try:
        # --------- Step 1: create visualization document in memory with PDFlib --------
        p = PDFlib()

        # create a new PDF document in memory
        p.begin_document("", "")

        p.set_info("Creator", "dynamicsign.py")
        p.set_info("Author", "Rainer Ploeckl")
        p.set_info("Title", "Dynamic signature visualization document")

        p.begin_page_ext(300, 50, "")
        # Prepare dynamic text for the signature visualization 
        timebuf = "Digitally signed %s at %s" % (time.strftime("%b %d, %Y", time.localtime()), time.strftime("%H:%M:%S", time.localtime()))
          
        textoptlist = "fontsize=10 fontname=Helvetica encoding=unicode fillcolor=red"
        p.fit_textline(timebuf, 10, 10, textoptlist)
        p.end_page_ext("")

        p.end_document("")

        # fetch the buffer containing the PDF document
        contents = p.get_buffer()

    except PDFlibException:
        print_exc(file=stdout)

except Exception:
    print_exc(file=stdout)

# ------------------ Step 2: sign the document -----------------
    
try:
    try:
        # create a new PLOP object
        plop = PLOP()

        optlist = "searchpath={%s}" % searchpath
        plop.set_option(optlist)

        # Create a virtual PVF file from the document data in memory
        plop.create_pvf("/pvf/visdoc.pdf", contents, "")
        
        doc = plop.open_document(argv[1], "")
        if (doc == -1):
            raise PLOPException("Error: %s" % (PLOP_get_errmsg(),
                                                     plop.get_errmsg()))


        # open input document from PVF file
        visdoc = plop.open_document("/pvf/visdoc.pdf", "")
        if (visdoc == -1):
            raise PLOPException("Error: %s" % (PLOP_get_errmsg(),
                                                     plop.get_errmsg()))

        # Add field option for visualization to base signature options
        optlist = "%s field={visdoc=%d rect={10 10 adapt adapt}} " % (sign_opts, visdoc)
        
        # sign the output file
        if (plop.prepare_signature(optlist) == -1):
            raise PLOPException("Error: %s: %s" % (plop.get_apiname(),
                                                     plop.get_errmsg()))

        optlist = "input={%d}" % doc
        if (plop.create_document(argv[2], optlist) == -1):
            raise PLOPException("Error: %s: %s" % (plop.get_apiname(),
                                                     plop.get_errmsg()))

        # close the input document
        plop.close_document(doc, "")

        # close the visualization document
        plop.close_document(visdoc, "")

        # delete PVF file containing input document
        if (plop.delete_pvf("/pvf/visdoc.pdf") == -1):
            raise PLOPException("Error: %s" % plopget_errmsg())

    except PLOPException as ex:
	print("PDFlib PLOP exception occurred:")
	print("[%d] %s: %s" % (ex.errnum, ex.apiname, ex.errmsg))

except Exception as ex:
    print(ex)

finally:
    if plop:
        plop.delete()
    if p:
        p.delete()
